package cn.ninegame.genericframework.module;

import android.content.Context;
import android.util.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import cn.ninegame.genericframework.basic.CacheManager;
import cn.ninegame.genericframework.basic.ExceptionManager;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.OnCacheCleanListener;
import cn.ninegame.genericframework.basic.RuntimeVariables;
import cn.ninegame.genericframework.basic.SecurityManager;
import cn.ninegame.genericframework.basic.StatManager;
import cn.ninegame.genericframework.config.DynamicLoadConfig;
import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.exception.DeleteFileException;
import cn.ninegame.genericframework.exception.DiskFullException;
import cn.ninegame.genericframework.tools.ApkUtils;
import cn.ninegame.genericframework.tools.FileLocker;
import cn.ninegame.genericframework.tools.FileUtil;
import cn.ninegame.genericframework.tools.IOUtils;
import cn.ninegame.genericframework.tools.LOG;
import cn.ninegame.genericframework.tools.ZipUtil;
import cn.ninegame.genericframework.tools.dynamicload.DexLoader;
import cn.ninegame.genericframework.tools.dynamicload.NativeLibManager;

import static cn.ninegame.genericframework.tools.ZipUtil.unZipDir;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * ModuleLoader, 用于加载Module
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/2/5
 */
public class ModuleLoader {
    private final static String TAG = "ModuleLoader";
    // 待加载apk所处位置， 可能是 lib目录， 或者  modules/packageName/version 目录
    public static final String MODULE_APK_DIR = "module_apk_dir";
    // 待加载apk所在的 lib 目录
    public static final int DIR_LIB = 0;
    // 待加载apk所在的 modules 目录
    public static final int DIR_MODULE = 1;
    // 龙卷风目前支持的 api
    private static final Integer[] CYCLONE_SUPPORT_API_LIST = {15, 16, 17, 18, 22};
    // framework apk 所在路径
    private static File sFrameworkApkFile;

    private SecurityManager mSecurityManager;
    private CacheManager mCacheManager;
    private FileLocker mLoadModuleLocker;
    private ModulePathManager mModulePathManager;
    private int mCurrentModuleDirType = -1;
    private boolean mIsVersionFirst;
    /**
     * 尝试清理缓存(整个生命周期只清理一次)
     */
    private static boolean sIsClean = false;
    private File sFrameworkApkFile1;

    public interface State{
        public static final int LOAD_SUCCESS = 0;

        //常规执行链的状态值必须大于 0
        public static final int PREPARE_MODULE_FOLDER = 1;     //创建所需要的目录
        public static final int PREPARE_MODULE_APK = 2;            //从assets拷贝module.apk到data/data
        public static final int EXTRACT_FILE = 3;                              //解压so和data等文件
        public static final int PARSE_UPDATE_MODULE_INFO_IF_NEED = 4;       //解析更新manifest总表（只有第一次加载module.apk的时候才需要）
        public static final int LOAD_DEX = 5;                                    //dex 加载
        public static final int DELETE_OLD_VERSION = 7;   // 删除该模块下面的旧版本数据

        //异常状态的值都必须小于 0
        public static final int DELETE_FILE_ERROR = -1;
        public static final int LOAD_DEX_ERROR = -2;
        public static final int MAKE_DIR_FAIL_ERROR = -3;
        public static final int EXTRACT_FILE_FAIL_ERROR = -4;
        public static final int PARSER_MANIFEST_ERROR = -5;
        public static final int EXTRACT_APK_TO_MODULE_FAIL_ERROR = -6;   // 复制apk文件到module目录失败
    }

    private int mCurrentState;
    private boolean mRetried = false;

    public ModuleLoader(){
        mSecurityManager = FrameworkService.getInstance().getService(SecurityManager.class);
        mCacheManager = FrameworkService.getInstance().getService(CacheManager.class);
        mModulePathManager = ModulePathManager.getInstance();
    }

    public void load(Context context,ModuleInfo moduleInfo) throws Exception {
        if(moduleInfo.isAndroidLibrary){
            mCurrentState =State.LOAD_SUCCESS;
            return;
        }
        mLoadModuleLocker = new FileLocker(mModulePathManager.getModuleLockPath(context, moduleInfo.getID()));
        mLoadModuleLocker.lock(true);
        Log.d(TAG, moduleInfo.getID() + " get the module lock");
        mCurrentState = State.DELETE_OLD_VERSION;
        execute(context, moduleInfo);
    }

    private void cleanForMoreSpace() {
        if (mCacheManager == null || sIsClean) {
            return;
        }
        sIsClean = true;
//        final long before = StorageUtil.getAvailInternalStorageSizeInKiloBytes();
        mCacheManager.cleanCache(new OnCacheCleanListener() {
            @Override
            public void onCacheClean(boolean isSuccess) {
//                long after = StorageUtil.getAvailInternalStorageSizeInKiloBytes();
//                Log.i(TAG, "clean result:" + isSuccess + ", free:" + (after - before) + "kb");
            }
        });
    }

    private void execute(Context context, ModuleInfo moduleInfo) throws Exception {
        LOG.i(TAG, moduleInfo.getID() + ":load module start,pid:" + android.os.Process.myPid());

        long start = System.currentTimeMillis();

        mIsVersionFirst = !mModulePathManager.getModuleManifest(moduleInfo.getID(), moduleInfo.getVersion()).exists();
        while (mCurrentState > 0 ){
            switch (mCurrentState) {
                case State.DELETE_OLD_VERSION:
                    deleteOldVersion(moduleInfo);
                    break;
                case State.PREPARE_MODULE_FOLDER:
                    prepareModuleFolderTree(moduleInfo);
                    break;
                case State.PREPARE_MODULE_APK:
                    prepareModuleApk(context, moduleInfo);
                    break;
                case State.EXTRACT_FILE:
                    extractFiles(moduleInfo);
                    break;
//                case State.PARSE_UPDATE_MODULE_INFO_IF_NEED:
//                    parse2UpdateModuleInfoIfNeed(moduleInfo);
//                    break;
                case State.LOAD_DEX:
                    loadDexImpl(context, moduleInfo);
                    break;
            }
        }

        mLoadModuleLocker.release();
        long consume = System.currentTimeMillis() - start;
        LOG.i(TAG, moduleInfo.getID() +":load dex end, consume:" + consume);

        if (mCurrentState == State.LOAD_SUCCESS){
            // 加载成功
            HashMap data = new HashMap<String, String>();
            data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_LOAD_MODULE_DEX_JANK);
            data.put(StatManager.Constant.KEY_MODULE_ID, moduleInfo.getID());
            data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(consume));
            data.put(StatManager.Constant.KEY_MODULE_VERSION_FIRST, String.valueOf(mIsVersionFirst));
            FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
        }else{
            handleException(mCurrentState, moduleInfo);
        }
    }

    private void handleException(int state, ModuleInfo moduleInfo) throws Exception {
//        Log.i(TAG, "handleException, state = " + state);
        switch (state){
            case State.DELETE_FILE_ERROR:
                // 删除被篡改文件失败
                throw new DeleteFileException("can not delete file which has bean modified!");
            case State.LOAD_DEX_ERROR:
                // Dex 加载失败
            case State.EXTRACT_FILE_FAIL_ERROR:
                // 解压文件失败
            case State.MAKE_DIR_FAIL_ERROR:
                // 创建文件夹失败

                if (ExceptionManager.isDiskFullException()){
                    //属于磁盘已满引起的异常
                    stat(StatManager.Constant.KEY_FULL_DISK_ERROR, moduleInfo.getID());

                    cleanForMoreSpace();
                    throw new DiskFullException(moduleInfo, "load module fail because of disk is full");
                } else {
                    stat(StatManager.Constant.KEY_UNKNOW_ERROR, moduleInfo.getID());
                }
                break;
        }
    }

    /**
     * 解压so和data等文件
     */
    private void extractFiles(ModuleInfo moduleInfo){
//        Log.i(TAG, "extract module files");
        String dexName = moduleInfo.getID();
        String version = moduleInfo.getVersion();

        File moduleApk  = getCurrentModuleApk(getModuleApkDirType(dexName, version), dexName, version);
        boolean manifestExist = mModulePathManager.getModuleManifest(dexName, version).exists();
        boolean libsoExist = mModulePathManager.getModuleSODir(dexName, version).exists();
        String[] primaryCpuAbi = NativeLibManager.abis;
        if (!manifestExist || !libsoExist) {
            //module的 manifest 或者 so 目录不存在的时候才做下面的东西，测试发现 new ZipFile有一定的耗时
            try {
                ZipFile zipFile = new ZipFile(moduleApk);
                extractManifest(zipFile, dexName, version);
                //1.1.0屏蔽,1.1.1将会打开,提取高达插件中的so文件.
                if(null!=primaryCpuAbi && primaryCpuAbi.length>0){
                    extractLibso(zipFile, dexName, version,primaryCpuAbi);
                }else{
                    extractLibso(zipFile, dexName, version);
                }
                zipFile.close();
            }catch (Exception e){
                e.printStackTrace();
                stat(StatManager.Constant.KEY_EXTRACT_FILE_FAIL, dexName);
                mCurrentState = State.EXTRACT_FILE_FAIL_ERROR;
                return;
            }
        }

        setNextNormalState(State.LOAD_DEX);
    }

    /**
     * 删除旧版本和无用文件
     * @param moduleInfo
     */
    private void deleteOldVersion(ModuleInfo moduleInfo){
        ModulePathManager modulePathManager = mModulePathManager;
        String dexName = moduleInfo.getID();
        File rootDir = modulePathManager.getModuleRootDir(dexName);
        String dataDir = modulePathManager.getModuleDataDirPath(dexName);

        ModuleSignature moduleSignature =  new ModuleSignature(moduleInfo.getID());
        if (rootDir.exists()){
            // module 所属目录已经存在
            String signature = moduleSignature.getSignature();
            if (signature == null || !moduleInfo.getSignature().equals(signature)) {
                // module 签名信息不存在 或 签名不一致
                Log.i(TAG, "deleteOldVersion, deleting old version data");
                moduleSignature.updateSignature(moduleInfo.getSignature());

                File[] files = rootDir.listFiles();
                //遍历该Module的目录， 除了当前版本数据目录和data目录， 其他的全部清除
                if (files != null) {
                    for (File f : files) {
                        if (!f.getPath().equals(dataDir)) {
                            //覆盖安装下， 只保留 data 目录， 其他全部删除
                            boolean success = FileUtil.deleteDirectory(f);
                            Log.d(TAG, "deleteOldVersion, path = " + f.getPath() + ",  ret = " + success);
                        }
                    }
                }
            }
        } else {
            // module 所属目录不存在，为首次安装
            moduleSignature.updateSignature(moduleInfo.getSignature());
        }
        setNextNormalState(State.PREPARE_MODULE_FOLDER);
    }

    /**
     * 解压manifest到moduleName/version/目录下
     */
    private void extractManifest(ZipFile zipFile, String dexName, String version) throws IOException{
        File manifestFile = mModulePathManager.getModuleManifest(dexName, version);
        if(!manifestFile.exists()) {
            ZipEntry zipEntry = zipFile.getEntry(DynamicLoadConfig.MODULE_MANIFEST_ENTRY);
            if (zipEntry != null) {
                boolean isSuccess = ZipUtil.extract(zipFile, zipEntry, manifestFile);
                if(!isSuccess){
                    FileUtil.deleteFiles(manifestFile);
                }
            }
        }
    }

    /**
     * 解压module私有文件到data目录
     * @param zipFile
     * @param dexName
     * @param version
     * @throws IOException
     */
//    private void extractData(ZipFile zipFile, String dexName, String version) throws IOException {
//        //data
//        File dataDir = mModulePathManager.getModuleDataDir(dexName);
//        if(!dataDir.exists()){
//            dataDir.mkdirs();
//
//            String unzipDir = mModulePathManager.getModuleDataDirPath(dexName);
//            ZipUtil.unZipDir(zipFile, DynamicLoadConfig.MODULE_FILES_ENTRY, unzipDir);
//        }
//    }

    /**
     * 解析更新manifest总表（只有第一次加载module.apk的时候才需要）
     * @param moduleInfo
     */
    private void parse2UpdateModuleInfoIfNeed(ModuleInfo moduleInfo) {
        String dexName = moduleInfo.getID();
        String version = moduleInfo.getVersion();

        File manifestFile = mModulePathManager.getModuleManifest(dexName, version);

        if(manifestFile.exists()){
            try {
//                Log.i(TAG, "parse manifest to update the main table");
                ArrayList<Module> modulesList = parseModuleManifest(manifestFile.getPath());
                updateAndSaveModuleManifest(modulesList);
            }catch (Exception e){
                e.printStackTrace();
                retryIfNeed(State.PARSER_MANIFEST_ERROR);
                return ;
            }
        }

        setNextNormalState(State.LOAD_DEX);
    }

    /**
     * 更新总表和保存（//FIXME lihq 又要重新读取总表）
     * @param modulesList
     */
    private void updateAndSaveModuleManifest(ArrayList<Module> modulesList) {
        if(modulesList != null && modulesList.size() >0){

        }
    }

    /**
     * 解析manifest(//FIXME lihq json加解密)
     * @param path
     */
    private ArrayList<Module> parseModuleManifest(String path) {

        ArrayList<Module> moduleList = null;

        File manifest = new File(path);
        if(manifest.exists()){
            InputStream inputStream = null;
            try{
                inputStream = new FileInputStream(manifest);
                moduleList = new JsonModuleManifestParser().parserForModule(inputStream);
            }catch (Exception e){
                e.printStackTrace();
            }finally {
                IOUtils.safeClose(inputStream);
            }
        }

        return moduleList;
    }

    /**
     * 解压出so文件
     */
    private void extractLibso(ZipFile zipFile,String dexName, String version) throws IOException {
        //so
        File soDir = mModulePathManager.getModuleSODir(dexName, version);
        if(!soDir.exists()){
            soDir.mkdirs();
            String unzipDir = mModulePathManager.getModuleSODirPath(dexName, version);
            //只解压当前手机支持的cpuAibus
            unZipDir(zipFile, DynamicLoadConfig.MODULE_SO_LIBS_ENTRY, unzipDir);
        }
    }


    /**
     * 解压出so文件
     */
    private void extractLibso(ZipFile zipFile,String dexName, String version,String cpuAbis[]) throws IOException {
        //so
        File soDir = mModulePathManager.getModuleSODir(dexName, version);
        if(!soDir.exists() || (soDir.exists() && (null!=soDir.list() || soDir.list().length==0))){
            soDir.mkdirs();
            String unzipDir = mModulePathManager.getModuleSODirPath(dexName, version);
            //只解压当前手机支持的cpuAibus
            for(String cpu: cpuAbis){
                int unZipCount = ZipUtil.unZipDir(zipFile, DynamicLoadConfig.MODULE_SO_LIBS_ENTRY + "/"+cpu, unzipDir);
                if(unZipCount>0) break;
            }
        }
    }

    /**
     * 解压manifest到moduleName/version/目录下
     */
    private void extractManifest(File moduleApk,File extractDir) throws IOException{
        ZipFile zipFile = new ZipFile(moduleApk);
        ZipEntry zipEntry = zipFile.getEntry(mModulePathManager.getManifestFileName());
        if(zipEntry != null){
            ZipUtil.extract(zipFile, zipEntry, extractDir);
        }
    }

    /**
     * 解析子模块的manifest.json
     * @param dexName
     * @param version
     * @throws Exception
     */
    private void prepareModuleManifest(String dexName, String version) throws Exception{
        File moduleApk = mModulePathManager.getModuleApk(dexName, version);
        if(moduleApk != null && moduleApk.exists()){
            String rootPath = moduleApk.getParent();
            File rootDir = new File(rootPath);
            if(!rootDir.exists()){
                rootDir.mkdirs();
            }
            File manifestFile =  mModulePathManager.getModuleManifest(dexName,version);
            if(!manifestFile.exists()){
                extractManifest(moduleApk,rootDir);
            }
        }
    }

    /**
     * 创建所需要的目录
     * @param moduleInfo
     */
    private void prepareModuleFolderTree(ModuleInfo moduleInfo){
//        Log.i(TAG, "build module folder tree");

        String moduleName = moduleInfo.getID();
        String moduleVersion = moduleInfo.getVersion();

        boolean bMakeDirSuccess = true;

        //root
        String moduleRootPath = mModulePathManager.getModuleRootDirPath(moduleName);
        File moduleRootDir = new File(moduleRootPath);
        if(!moduleRootDir.exists()){
            bMakeDirSuccess &= moduleRootDir.mkdirs();
        }

        //version
        File versionDir =  mModulePathManager.getModuleVersionDir(moduleName,moduleVersion);
        if(!versionDir.exists()){
            bMakeDirSuccess &= versionDir.mkdirs();
        }

        //code_cache
        File codeCacheDir = mModulePathManager.getModuleOdexDir(moduleName, moduleVersion);
        if(!codeCacheDir.exists()){
            bMakeDirSuccess &= codeCacheDir.mkdirs();
        }

        //data
        File dataDir = mModulePathManager.getModuleDataDir(moduleName);
        if(!dataDir.exists()) {
            bMakeDirSuccess &= dataDir.mkdirs();
        }

        if (bMakeDirSuccess) {
            if (mRetried){
                stat(StatManager.Constant.KEY_PREPARE_MODULE_FOLDER_TREE_RETRY_SUCCESS, moduleName);
            }

            setNextNormalState(State.PREPARE_MODULE_APK);
        }else{
            //创建文件夹失败
            if (!mRetried) {
                stat(StatManager.Constant.KEY_PREPARE_MODULE_FOLDER_TREE_FAIL, moduleName);
            }
            retryIfNeed(State.MAKE_DIR_FAIL_ERROR);
        }
    }

    /**
     * 从assets拷贝module.apk到data/data
     * assets目录结构和解压出来的目录结构，可由外部定义
     * @param context
     * @param moduleInfo
     */
    private void prepareModuleApk(Context context, ModuleInfo moduleInfo){
//        Log.i(TAG, "prepare module apk");
        String dexName = moduleInfo.getID();
        String version = moduleInfo.getVersion();
        boolean isPlugin = moduleInfo.isPlugin();

        int curApkDirType = getModuleApkDirType(dexName, version);
        File moduleApk = getCurrentModuleApk(curApkDirType, dexName, version);

        if (moduleApk.exists()) {
            // apk存在
            if ((ApkUtils.isRootSystem() && !FrameworkService.getInstance().IsDebug() && (!validatePublicKey(moduleApk.getPath()) || !compareApkSignature(moduleApk, moduleInfo.getSignature())))) {
                // 设备已root 且 apk已被篡改,验证签名和crc值.
                if (DIR_MODULE == curApkDirType) {
                    // apk 在 modules 目录
                    stat(StatManager.Constant.KEY_MODULE_APK_NO_AVAILABLE, dexName);
                    if (!moduleApk.delete()) {
                        //删除文件失败
                        mCurrentState = State.DELETE_FILE_ERROR;
                        stat(StatManager.Constant.KEY_MODULE_APK_DELETE_FAIL, dexName);
                    }
                } else {
                    // apk 在 lib 目录
                    stat(StatManager.Constant.KEY_LIB_APK_NO_AVAILABLE, dexName);
                }

                fixModuleApkFromLib(context, dexName, version,isPlugin);
            } else if (DIR_LIB == curApkDirType && isMt65()) {
                // mt65xx 联发科的cpu机器， 不能加载 so格式的 jar
                fixModuleApkFromLib(context, dexName, version,isPlugin);
            }else{
                // apk 存在 且 通过校验
                // 进入加载解压配置文件状态
                if (mRetried) {
                    stat(StatManager.Constant.KEY_EXTRACT_MODULE_APK_RETRY_SUCCESS, dexName);
                }
                setNextNormalState(State.EXTRACT_FILE);
            }
        } else {
            // apk 不存在
            if (DIR_MODULE == curApkDirType) {
                // module 指向的apk位置在 module目录， 找不到， 说明被异常删除了
                stat(StatManager.Constant.KEY_MODULE_APK_MISS, dexName);
            }

            fixModuleApkFromLib(context, dexName, version,isPlugin);
        }
    }

    private boolean validatePublicKey(String apkFile){
        String[] keys = ApkUtils.getApkPublicKey(apkFile);
        return Arrays.equals(keys, RuntimeVariables.publicKeys);
    }

    private void fixModuleApkFromLib(Context context, String dexName, String version,boolean isPlugin){
        // 从apk lib 目录拷贝 so格式的module apk
        if (extractModuleApkFromLib(context, dexName, version,isPlugin)) {
            mCurrentModuleDirType = DIR_MODULE;
            FileUtil.writeProperty(mModulePathManager.getModulePropertyFile(dexName, version), MODULE_APK_DIR, String.valueOf(DIR_MODULE));
        } else {
            //从apk lib 目录拷贝 so格式的module apk 失败
            stat(StatManager.Constant.KEY_UNZIP_APK_FROM_LIB_FAIL, dexName);
            File file = mModulePathManager.getModuleApk(dexName, version);
            if (file.exists()) {
                //如果有临时文件则删除
                file.delete();
            }
            retryIfNeed(State.EXTRACT_APK_TO_MODULE_FAIL_ERROR);
        }
    }

    /**
     * 对比文件签名的crc32值
     * @param file
     * @param signature
     * @return true表示crc32值相等
     */
    private boolean compareApkSignature(File file, String signature) {
        String fileCrc32 = "";
        try {
            fileCrc32 = String.valueOf(ZipUtil.getZipCrc(file));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return fileCrc32.equals(signature);
    }

    private IModuleManifest parseModuleInfo(File file){
        if(file == null){
            return null;
        }
        IModuleManifest moduleManifest = null;
        InputStream is = null;
        try {
            is = new FileInputStream(file);
            IModuleManifest[] moduleManifestArray = new JsonModuleManifestParser().parser(is);
            moduleManifest = moduleManifestArray[0];
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            IOUtils.safeClose(is);
        }
        return moduleManifest;
    }

    /**
     * 具体实现Dex的加载（dexopt）
     * @param context
     * @param moduleInfo
     */
    private void loadDexImpl(Context context, ModuleInfo moduleInfo){
        Log.i(TAG, "load dex implement");
        mLoadModuleLocker.release();
        Log.d(TAG, moduleInfo.getID() + " release the module lock");
        String dexName = moduleInfo.getID();
        String version = moduleInfo.getVersion();

        int moduleDirType = getModuleApkDirType(dexName, version);
        File moduleApk = getCurrentModuleApk(moduleDirType, dexName, version);
        File odexDir = mModulePathManager.getModuleOdexDir(dexName,version);
        //1.1.0屏蔽,1.1.1将会打开
        File odexSoFile = mModulePathManager.getModuleSODir(dexName,version);
        boolean bOpenHookSuccess = false;
        try {
            //ClassLoader loader = ModuleLoader.class.getClassLoader();
            //修复双层类加载问题的.
            ClassLoader loader = context.getClassLoader();
            ArrayList<File> dexList = new ArrayList<File>();
            dexList.add(moduleApk);
            //一定要加在第二个, 因为第一个installDexes在用,去掉这一行,似乎对加载没有影响(泓人额外加入的测试代码,可能忘记去掉了)
            //dexList.add(mModulePathManager.getSelfApkFile());
            DexLoader.installDexes(loader, odexDir, dexList);
            if (mRetried){
                stat(StatManager.Constant.KEY_LOAD_MODULE_RETRY_SUCCESS, dexName);
            }
            setNextNormalState(State.LOAD_SUCCESS);
            //加载高达子模块的so文件路径到ClassLoader中,1.1.0屏蔽,1.1.1将会打开
            try{
                NativeLibManager.injectHackNativeLib(odexSoFile,loader);
            }catch (Exception ex){
                ex.printStackTrace();
            }
        }catch (Exception e){
            e.printStackTrace();
            if ((moduleDirType == DIR_LIB) && !ExceptionManager.isDiskFullException()) {
                //加载lib目录下的module失败， 并且当前非磁盘已满
                fixModuleApkFromLib(context, dexName, version,moduleInfo.isPlugin);
                setNextNormalState(State.EXTRACT_FILE);
                return;
            }
            retryIfNeed(State.LOAD_DEX_ERROR);
        }
    }
    /**
     * 如果没有重试过， 则不改变状态，让 execute 方式重试
     * 如果重试过，则说明已经失败2次，设置对应的异常状态，让 execute方法中止加载
     * @param errorType 异常类型
     */
    private void retryIfNeed(int errorType){
        if (mRetried){
            mCurrentState = errorType;
            mRetried = false;
        }else{
            mRetried = true;
        }
    }

    /**
     * 进入下一个状态，进去前将共享的重试标志复位
     * @param state
     */
    private void setNextNormalState(int state){
        mCurrentState = state;
        mRetried = false;
    }

    private void stat(String value, String moduleId){
        HashMap<String, String> data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_LOAD_MODULE_TYPE, value);
        data.put(StatManager.Constant.KEY_MODULE_ID, moduleId);

        FrameworkService.addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);
    }

    /**
     * 获取 module apk 路径的类型， 类型有下面2种
     * {@link #DIR_LIB}， lib 目录
     * {@link #DIR_MODULE}    modules/packageName/version  目录
     * @return
     */
    public int getModuleApkDirType(String moduleID, String version){
        if (mCurrentModuleDirType == -1) {
            File propertyFile = mModulePathManager.getModulePropertyFile(moduleID, version);
            if (!propertyFile.exists()) {
                try {
                    propertyFile.createNewFile();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            String moduleApkDirTypeStr = FileUtil.readProperty(propertyFile, MODULE_APK_DIR);
            if (moduleApkDirTypeStr == null || "".equals(moduleApkDirTypeStr)) {
                mCurrentModuleDirType = DIR_LIB;
                FileUtil.writeProperty(propertyFile, MODULE_APK_DIR, String.valueOf(mCurrentModuleDirType));
            } else {
                mCurrentModuleDirType = Integer.valueOf(moduleApkDirTypeStr);
            }
        }
        return mCurrentModuleDirType;
    }

    /**
     *  获取当前待加载的apk
     * @param type
     * @param moduleID
     * @param version
     * @return
     */
    private File getCurrentModuleApk(int type, String moduleID, String version){
        File moduleApk;
        if (DIR_LIB == type) {
            moduleApk = mModulePathManager.getModuleApkInLib(moduleID);
        } else {
            moduleApk = mModulePathManager.getModuleApk(moduleID, version);
        }

        return moduleApk;
    }

    /**
     * 从原APK中的 lib/armeabi/ 目录解压出Module Apk
     *
     * @param context
     * @param moduleID
     * @param version
     * @return
     */
    private boolean extractModuleApkFromLib(Context context, String moduleID, String version,boolean isPlugin){
        File moduleApk= mModulePathManager.getModuleApk(moduleID, version);
        if (!moduleApk.exists()) {
            String moduleFileNameInLib = mModulePathManager.getModuleApkAbsolutePath(moduleID);
            File apkFile;
            try {
                if(isPlugin){
                    apkFile = mModulePathManager.getSelfRootApkFile();
                }else{
                    apkFile = mModulePathManager.getSelfApkFile();
                }
            } catch (Exception e) {
                stat(StatManager.Constant.GET_FRAMEWORK_APK_PATH_FAIL, moduleID);
                return false;
            }
            boolean ret = ZipUtil.unZipFile(apkFile.getAbsolutePath(), moduleFileNameInLib, moduleApk);
            if (ret) {
                mSecurityManager.initDynamicFileSignature(moduleApk.getPath());
            }
            return ret;
        }
        return true;
    }

    /**
     * cpu架构是否数据 联发科 mt65xx 系列
     * @return
     */
    private boolean isMt65(){
        return android.os.Build.HARDWARE.toLowerCase().contains("mt65");
    }
}
